<?php

namespace Drupal\localgov_entity_share\Plugin\EntityShareClient\Processor;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\entity_share_client\ImportProcessor\ImportProcessorPluginBase;
use Drupal\entity_share_client\RuntimeImportContext;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Filters out LocalGov parent reference fields from relationships.
 *
 * This prevents them from being followed by the Entity Reference processor
 * and prevents circularity, as these fields have a corresponding child
 * reference field pointing in the opposite direction.
 *
 * The value of the reference field is not affected.
 *
 * @ImportProcessor(
 *   id = "localgov_reference_field_filter",
 *   label = @Translation("Localgov parent reference field filter"),
 *   description = @Translation("Prevents LocalGov-related parent reference fields from being followed by the Entity Reference processor."),
 *   stages = {
 *     "prepare_entity_data" = 0,
 *   },
 * )
 */
class LocalgovReferenceFieldFilter extends ImportProcessorPluginBase {

  /**
   * The LocalGov reference fields to filter out.
   */
  const LOCALGOV_FILTERED_FIELDS = [
    'node' => [
      'localgov_services_parent',
      'localgov_step_parent',
      'localgov_guides_parent',
      'localgov_subsites_parent',
    ],
  ];

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_field.manager'),
    );
  }

  /**
   * Creates a LocalgovReferenceFieldFilter instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityFieldManagerInterface $entity_field_manager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareEntityData(RuntimeImportContext $runtime_import_context, array &$entity_json_data) {
    $field_mappings = $runtime_import_context->getFieldMappings();
    [$entity_type_id, $entity_bundle] = explode('--', $entity_json_data['type']);

    // Unset all our defined entity reference fields from the JSONAPI
    // relationships array.
    foreach (static::LOCALGOV_FILTERED_FIELDS[$entity_type_id] as $field_name) {
      $field_public_name = $field_mappings[$entity_type_id][$entity_bundle][$field_name];

      unset($entity_json_data['relationships'][$field_public_name]);
    }
  }

}
